local addonName = ...
local VA = CreateFrame('Frame')
local BetterDate = BetterDate
local C_VignetteInfo_GetVignettes = C_VignetteInfo.GetVignettes
local C_VignetteInfo_GetVignettePosition = C_VignetteInfo.GetVignettePosition
local C_VignetteInfo_GetVignetteInfo = C_VignetteInfo.GetVignetteInfo
local mapID = C_Map.GetBestMapForUnit("player")
local GetInstanceInfo = GetInstanceInfo
local format, find, tonumber, strmatch = string.format, string.find, tonumber, strmatch
local styledAddonName = GetAddOnMetadata(addonName, 'Title')
VA.trackVignettes, VA.recent = true, {}

VA.IsGarrison = {
	[1152] = true,
	[1153] = true,
	[1154] = true,
	[1158] = true,
	[1159] = true,
	[1160] = true,
	[1330] = true,
	[1331] = true,
}
VA.AtlasBlacklist = {
	["QuestNormal"] = true,
	["QuestObjective"] = true,
}

VA.AtlasWhitelist = {
	["VignetteKill"] = true,
	["VignetteLoot"] = true,
	["VignetteEvent"] = true,
	["VignetteEventElite"] = true,
}

function VA:FormatAtlasString(str)
	local atlasInfo = C_Texture.GetAtlasInfo(str)
	if (not atlasInfo) then return "" end
	local file = atlasInfo.filename or atlasInfo.file;
	local atlasWidth = atlasInfo.width / (atlasInfo.rightTexCoord - atlasInfo.leftTexCoord);
	local atlasHeight = atlasInfo.height / (atlasInfo.bottomTexCoord - atlasInfo.topTexCoord);
	local pxLeft = atlasWidth * atlasInfo.leftTexCoord;
	local pxRight = atlasWidth * atlasInfo.rightTexCoord;
	local pxTop = atlasHeight * atlasInfo.topTexCoord;
	local pxBottom = atlasHeight * atlasInfo.bottomTexCoord;
	local size = 16
	return format("|T%s:%d:%d:0:0:%d:%d:%d:%d:%d:%d|t", file, size, size, atlasWidth, atlasHeight, pxLeft, pxRight, pxTop, pxBottom);
end

function VA:GetVignetteInfo(vignetteGUID)
	local vInfo = C_VignetteInfo_GetVignetteInfo(vignetteGUID);
	if vInfo then
		if vInfo.onWorldMap and not vInfo.onMinimap and vInfo.type ~= 0 then return end

		local name, atlasName, vignetteID = vInfo.name, vInfo.atlasName, vInfo.vignetteID
		if not self.AtlasWhitelist[atlasName] then return end

		local now = GetTime()
		local instanceID = select(8, GetInstanceInfo())
		if not (self.IsGarrison[instanceID] or find(name,"Aguri") or name == "") then
			if self.recent[vignetteGUID] and now - self.recent[vignetteGUID] < 400 then self.recent[vignetteGUID] = now return end
			self.recent[vignetteGUID] = now
			--if not (AtlasWhitelist[atlasName] or AtlasBlacklist[atlasName]) then
			--	print("|cffEEE4AEVignetteAnouncer:|r Unknown (|cffFFE6E7"..atlasName.."|r), ID (|cffFFE6E7"..vignetteID.."|r)")
			--end
			--print("|cffEEE4AEVignetteAnouncer:|r Debug (|cffFFE6E7"..name.."|r), ID (|cffFFE6E7"..vignetteID.."|r)")

			if not name then name = vignetteID end
			mapID = C_Map.GetBestMapForUnit("player")
			local x, y
			if mapID then
				local vignettePosition = C_VignetteInfo_GetVignettePosition(vignetteGUID, mapID)
				if vignettePosition then
					x = tonumber(vignettePosition.x)
					y = tonumber(vignettePosition.y)
				end

				PlaySound(3175, "Master")
				local date = string.gsub(BetterDate(CHAT_TIMESTAMP_FORMAT or '%H:%M', time()), "(.-) $", "%1")
				local msg = self:FormatAtlasString(atlasName).." |cffFFFFFF"..name.."|r "..date
				RaidNotice_AddMessage( RaidBossEmoteFrame, msg, ChatTypeInfo["RAID_BOSS_EMOTE"] );
				if msg then
					if vignettePosition then
						DEFAULT_CHAT_FRAME:AddMessage(msg.."|cff85DBF3|HVAN:"..x.." "..y.." "..mapID.." "..name.."|h ["..format("%.2f",x *100)..", "..format("%.2f",y *100).."]|h|r")
					else
						DEFAULT_CHAT_FRAME:AddMessage(msg)
					end
				end
			end
		end
	end
end

function VA:UpdateVignettes()
	if UnitOnTaxi("player") then return end
	local vignetteGUIDs = C_VignetteInfo_GetVignettes();
	for i, vignetteGUID in ipairs(vignetteGUIDs) do
		if vignetteGUID then
			self:GetVignetteInfo(vignetteGUID)
		end
	end
end

function VA:OnEvent(e, id, ...)
	if e == "VIGNETTES_UPDATED" then
		if UnitOnTaxi("player") then return end
		self:UpdateVignettes()
	elseif e == "VIGNETTE_MINIMAP_UPDATED" and id then
		if UnitOnTaxi("player") then return end
		self:GetVignetteInfo(id)
	end
end

local SetHyperlink = _G.ItemRefTooltip.SetHyperlink
function ItemRefTooltip:SetHyperlink(data, ...)
	if (data):sub(1, 3) == "VAN" then
		local x, y, mapid, name = strmatch(data, "(%d+.%d+) (%d+.%d+) (%d+) %s?(.*)")
		if TomTom then
			TomTom:AddWaypoint(tonumber(mapid), tonumber(x), tonumber(y), { title = name.."\n"..styledAddonName, from = styledAddonName, minimap = false })
		else
			print("Enable TomTom for this feature.")
		end
	else
		SetHyperlink(self, data, ...)
	end
end

VA:SetScript("OnEvent", VA.OnEvent)
FrameUtil.RegisterFrameForEvents(VA, {
	"VIGNETTE_MINIMAP_UPDATED",
	"VIGNETTES_UPDATED",
})

function VA:Help(msg)
    local fName = "|cffEEE4AEVignette Announcer:|r "
    local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")
    if not cmd or cmd == "" or cmd == "help" then
        print(fName.."   |cff58C6FA/va|r")
        print("  |cff58C6FA/va auto    -|r  |cffEEE4AEAutomatically add Waypoints to map. (toggle)|r")
    elseif cmd == "all" then
        if VignetteAnnouncerDB.options.autoAddWaypoints then
            VignetteAnnouncerDB.options.autoAddWaypoints = false
            print(fName.."|cffB6B6B6Don't add waypoints to map automatically.")
        else
            VignetteAnnouncerDB.options.autoAddWaypoints = true
            print(fName.."|cff37DB33Add waypoints to map automatically.")
        end
    end
end

SLASH_VIGNETTEANNOUNCER1 = "/va"
SlashCmdList["VIGNETTEANNOUNCER"] = function(...)
    VA:Help(...)
end